@@ -12,12 +12,14 @@ import tempfile
 from time import time, sleep
 
 from tornado.ioloop import IOLoop
+from tornado.iostream import StreamClosedError
 from tornado import gen
 
 from .compatibility import JSONDecodeError
-from .core import Server, rpc, write
+from .core import Server, rpc, write, RPCClosed
+from .protocol import to_serialize
 from .utils import get_ip, ignoring, log_errors, tmpfile
-from .worker import _ncores, Worker, run
+from .worker import _ncores, Worker, run, TOTAL_MEMORY
 
 nanny_environment = os.path.dirname(sys.executable)
 
@@ -31,8 +33,8 @@ class Nanny(Server):
     """
     def __init__(self, scheduler_ip, scheduler_port, ip=None, worker_port=0,
                  ncores=None, loop=None, local_dir=None, services=None,
-                 name=None, memory_limit=None, environment=nanny_environment,
-                 quiet=False, **kwargs):
+                 name=None, memory_limit=TOTAL_MEMORY,
+                 environment=nanny_environment, quiet=False, **kwargs):
         self.ip = ip or get_ip()
         self.worker_port = None
         self._given_worker_port = worker_port
@@ -90,15 +92,19 @@ class Nanny(Server):
             yield gen.sleep(0.1)
 
         if self.process is not None:
+            process, self.process = self.process, None  # avoid race with _watch
             try:
                 # Ask worker to close
-                worker = rpc(ip='127.0.0.1', port=self.worker_port)
-                result = yield gen.with_timeout(
-                            timedelta(seconds=min(1, timeout)),
-                            worker.terminate(report=False),
-                            io_loop=self.loop)
+                with rpc(ip='127.0.0.1', port=self.worker_port) as worker:
+                    result = yield gen.with_timeout(
+                                timedelta(seconds=min(1, timeout)),
+                                worker.terminate(report=False),
+                                io_loop=self.loop)
+
             except gen.TimeoutError:
                 logger.info("Worker non-responsive.  Terminating.")
+            except StreamClosedError:
+                pass
             except Exception as e:
                 logger.exception(e)
 
@@ -118,9 +124,12 @@ class Nanny(Server):
                 logger.info("Nanny %s:%d failed to unregister worker %s:%d",
                         self.ip, self.port, self.ip, self.worker_port,
                         exc_info=True)
+            except (StreamClosedError, RPCClosed):
+                pass
             except Exception as e:
                 logger.exception(e)
 
+            self.process = process  # re-instantiate self.process
             if self.process:
                 with ignoring(OSError):
                     self.process.terminate()
@@ -193,8 +202,6 @@ class Nanny(Server):
                     except queues.Empty:
                         yield gen.sleep(0.1)
 
-
-
             logger.info("Nanny %s:%d starts worker process %s:%d",
                         self.ip, self.port, self.ip, self.worker_port)
             raise gen.Return('OK')
@@ -234,10 +241,13 @@ class Nanny(Server):
     @gen.coroutine
     def _close(self, stream=None, timeout=5, report=None):
         """ Close the nanny process, stop listening """
+        if self.status == 'closed':
+            raise gen.Return('OK')
         logger.info("Closing Nanny at %s:%d", self.ip, self.port)
         self.status = 'closed'
         yield self._kill(timeout=timeout)
-        self.scheduler.close_streams()
+        self.rpc.close()
+        self.scheduler.close_rpc()
         self.stop()
         raise gen.Return('OK')
 
