@@ -12,12 +12,17 @@ import tblib.pickling_support
 import tempfile
 from threading import Thread
 
+try:
+    import resource
+except ImportError:
+    resource = None
+
 from dask import istask
 from toolz import memoize, valmap
 from tornado import gen
 from tornado.iostream import StreamClosedError
 
-from .compatibility import Queue, PY3
+from .compatibility import Queue, PY3, PY2
 from .config import config
 
 logger = logging.getLogger(__name__)
@@ -33,6 +38,18 @@ def funcname(func):
         return str(func)
 
 
+def get_fileno_limit():
+    """
+    Get the maximum number of open files per process.
+    """
+    if resource is not None:
+        return resource.getrlimit(resource.RLIMIT_NOFILE)[0]
+    else:
+        # Default ceiling for Windows when using the CRT, though it
+        # is settable using _setmaxstdio().
+        return 512
+
+
 def get_ip(host='8.8.8.8', port=80):
     try:
         return [(s.connect((host, port)), s.getsockname()[0], s.close())
@@ -184,7 +201,10 @@ def key_split(s):
         s = s[0]
     try:
         words = s.split('-')
-        result = words[0].lstrip("'(\"")
+        if not words[0][0].isalpha():
+            result = words[0].lstrip("'(\"")
+        else:
+            result = words[0]
         for word in words[1:]:
             if word.isalpha() and not (len(word) == 8 and
                                        hex_pattern.match(word) is not None):
@@ -245,7 +265,6 @@ def ensure_ip(hostname):
 
 tblib.pickling_support.install()
 
-
 def get_traceback():
     exc_type, exc_value, exc_traceback = sys.exc_info()
     bad = [os.path.join('distributed', 'worker'),
@@ -473,6 +492,10 @@ def ensure_bytes(s):
     """
     if isinstance(s, bytes):
         return s
+    if isinstance(s, memoryview):
+        return s.tobytes()
+    if isinstance(s, bytearray) or PY2 and isinstance(s, buffer):
+        return bytes(s)
     if hasattr(s, 'encode'):
         return s.encode()
     raise TypeError(
