@@ -3,7 +3,7 @@ from __future__ import print_function, division, absolute_import
 from datetime import timedelta
 from importlib import import_module
 import logging
-from multiprocessing.pool import ThreadPool
+import multiprocessing
 import os
 import pkg_resources
 import tempfile
@@ -27,18 +27,25 @@ from tornado.iostream import StreamClosedError
 from .batched import BatchedSend
 from .utils_comm import pack_data, gather_from_workers
 from .compatibility import reload, unicode
-from .core import (rpc, Server, pingpong, dumps, loads, coerce_to_address,
+from .core import (rpc, Server, pingpong, coerce_to_address,
         error_message, read, RPCClosed)
+from .protocol.pickle import dumps, loads
 from .sizeof import sizeof
 from .threadpoolexecutor import ThreadPoolExecutor
 from .utils import funcname, get_ip, _maybe_complex, log_errors, All, ignoring
 
-_ncores = ThreadPool()._processes
+_ncores = multiprocessing.cpu_count()
 
 thread_state = local()
 
 logger = logging.getLogger(__name__)
 
+try:
+    import psutil
+    TOTAL_MEMORY = psutil.virtual_memory().total
+except ImportError:
+    logger.warn("Please install psutil to estimate worker memory use")
+    TOTAL_MEMORY = 8e9
 
 class Worker(Server):
     """ Worker Node
@@ -76,39 +83,51 @@ class Worker(Server):
         Auxiliary web servers running on this worker
     * **service_ports:** ``{str: port}``:
 
+    Parameters
+    ----------
+    scheduler_ip: str
+    scheduler_port: int
+    ip: str, optional
+    ncores: int, optional
+    loop: tornado.ioloop.IOLoop
+    local_dir: str, optional
+        Directory where we place local resources
+    name: str, optional
+    heartbeat_interval: int
+        Milliseconds between heartbeats to scheduler
+    memory_limit: int
+        Number of bytes of data to keep in memory before using disk
+    executor: concurrent.futures.Executor
+
     Examples
     --------
 
-    Create schedulers and workers in Python:
-
-    >>> from distributed import Scheduler, Worker
-    >>> c = Scheduler('192.168.0.100', 8786)  # doctest: +SKIP
-    >>> w = Worker(c.ip, c.port)  # doctest: +SKIP
-    >>> yield w._start(port=8786)  # doctest: +SKIP
-
-    Or use the command line::
+    Use the command line to start a worker::
 
         $ dask-scheduler
         Start scheduler at 127.0.0.1:8786
 
         $ dask-worker 127.0.0.1:8786
-        Start worker at:               127.0.0.1:8786
-        Registered with scheduler at:  127.0.0.1:8787
+        Start worker at:               127.0.0.1:1234
+        Registered with scheduler at:  127.0.0.1:8786
 
     See Also
     --------
-    distributed.scheduler.Scheduler:
+    distributed.scheduler.Scheduler
+    distributed.nanny.Nanny
     """
 
     def __init__(self, scheduler_ip, scheduler_port, ip=None, ncores=None,
                  loop=None, local_dir=None, services=None, service_ports=None,
-                 name=None, heartbeat_interval=1000, memory_limit=None, **kwargs):
+                 name=None, heartbeat_interval=5000, memory_limit=TOTAL_MEMORY,
+                 executor=None, **kwargs):
         self.ip = ip or get_ip()
         self._port = 0
         self.ncores = ncores or _ncores
         self.local_dir = local_dir or tempfile.mkdtemp(prefix='worker-')
         if not os.path.exists(self.local_dir):
             os.mkdir(self.local_dir)
+        self.memory_limit = memory_limit
         if memory_limit:
             try:
                 from zict import Buffer, File, Func
@@ -121,7 +140,7 @@ class Worker(Server):
             self.data = dict()
         self.loop = loop or IOLoop.current()
         self.status = None
-        self.executor = ThreadPoolExecutor(self.ncores)
+        self.executor = executor or ThreadPoolExecutor(self.ncores)
         self.scheduler = rpc(ip=scheduler_ip, port=scheduler_port)
         self.active = set()
         self.name = name
@@ -171,6 +190,16 @@ class Worker(Server):
                                                    io_loop=self.loop)
         self.loop.add_callback(self.heartbeat_callback.start)
 
+    def __str__(self):
+        return "<Worker: %s, threads: %d>" % (self.address, self.ncores)
+
+    __repr__ = __str__
+
+    @property
+    def worker_address(self):
+        """ For API compatibility with Nanny """
+        return self.address
+
     @gen.coroutine
     def heartbeat(self):
         if not self.heartbeat_active:
@@ -180,9 +209,9 @@ class Worker(Server):
                 yield self.scheduler.register(address=self.address, name=self.name,
                                         ncores=self.ncores,
                                         now=time(),
-                                        info=self.process_health(),
                                         host_info=self.host_health(),
                                         services=self.service_ports,
+                                        memory_limit=self.memory_limit,
                                         **self.process_health())
             finally:
                 self.heartbeat_active = False
@@ -211,6 +240,7 @@ class Worker(Server):
                         now=time(),
                         host_info=self.host_health(),
                         services=self.service_ports,
+                        memory_limit=self.memory_limit,
                         **self.process_health())
                 break
             except (OSError, StreamClosedError):
@@ -227,7 +257,9 @@ class Worker(Server):
 
     def identity(self, stream):
         return {'type': type(self).__name__, 'id': self.id,
-                'scheduler': (self.scheduler.ip, self.scheduler.port)}
+                'scheduler': (self.scheduler.ip, self.scheduler.port),
+                'ncores': self.ncores,
+                'memory_limit': self.memory_limit}
 
     @gen.coroutine
     def _close(self, report=True, timeout=10):
@@ -245,6 +277,7 @@ class Worker(Server):
 
         for k, v in self.services.items():
             v.stop()
+        self.rpc.close()
         self.status = 'closed'
         self.stop()
 
@@ -276,10 +309,8 @@ class Worker(Server):
             self.data.update(result)
             raise Return({'status': 'OK'})
 
-    def deserialize(self, function=None, args=None, kwargs=None, task=None):
+    def _deserialize(self, function=None, args=None, kwargs=None, task=None):
         """ Deserialize task inputs and regularize to func, args, kwargs """
-        if task is not None:
-            task = loads(task)
         if function is not None:
             function = loads(function)
         if args:
@@ -373,7 +404,7 @@ class Worker(Server):
 
         try:
             start = default_timer()
-            function, args, kwargs = self.deserialize(function, args, kwargs,
+            function, args, kwargs = self._deserialize(function, args, kwargs,
                     task)
             diagnostics['deserialization'] = default_timer() - start
         except Exception as e:
@@ -462,11 +493,12 @@ class Worker(Server):
     def compute_many(self, bstream, msgs, report=False):
         good, bad, data, num_transferred, diagnostics = yield self.gather_many(msgs)
 
+        if bad:
+            logger.warn("Could not find data for %s", sorted(bad))
+
         for msg in msgs:
             msg.pop('who_has', None)
 
-        if bad:
-            logger.warn("Could not find data for %s", sorted(bad))
         for k, v in bad.items():
             bstream.send({'status': 'missing-data',
                           'key': k,
@@ -492,7 +524,7 @@ class Worker(Server):
         diagnostics = dict()
         try:
             start = default_timer()
-            function, args, kwargs = self.deserialize(function, args, kwargs,
+            function, args, kwargs = self._deserialize(function, args, kwargs,
                     task)
             diagnostics['deserialization'] = default_timer() - start
         except Exception as e:
@@ -595,9 +627,7 @@ class Worker(Server):
         return run(self, stream, function=function, args=args, kwargs=kwargs)
 
     @gen.coroutine
-    def update_data(self, stream=None, data=None, report=True, deserialize=True):
-        if deserialize:
-            data = valmap(loads, data)
+    def update_data(self, stream=None, data=None, report=True):
         self.data.update(data)
         if report:
             response = yield self.scheduler.add_keys(
@@ -622,7 +652,7 @@ class Worker(Server):
         raise Return('OK')
 
     def get_data(self, stream, keys=None):
-        return {k: dumps(self.data[k]) for k in keys if k in self.data}
+        return {k: to_serialize(self.data[k]) for k in keys if k in self.data}
 
     def start_ipython(self, stream):
         """Start an IPython kernel
@@ -774,7 +804,7 @@ def dumps_task(task):
         elif not any(map(_maybe_complex, task[1:])):
             return {'function': dumps_function(task[0]),
                         'args': dumps(task[1:])}
-    return {'task': dumps(task)}
+    return to_serialize(task)
 
 
 def apply_function(function, args, kwargs, execution_state, key):
@@ -862,7 +892,7 @@ def convert_kwargs_to_str(kwargs, max_len=None):
         return "{{{}}}".format(", ".join(strs))
 
 
-from .protocol import compressions, default_compression
+from .protocol import compressions, default_compression, to_serialize
 
 # TODO: use protocol.maybe_compress and proper file/memoryview objects
 
@@ -908,6 +938,6 @@ def run(worker, stream, function=None, args=(), kwargs={}):
     else:
         response = {
             'status': 'OK',
-            'result': dumps(result),
+            'result': to_serialize(result),
         }
     raise Return(response)
